@@ -18,6 +18,9 @@
  * Date: Jan 29, 2013
  * Version 1.2 and newer
  *
+ * Modifications and updates by: Hasnain Lakhani
+ * Date: Mar 26, 2014
+ * Version 2.1
  */
 
 
@@ -77,7 +80,7 @@
 #endif
 
 #ifndef VERSION
-#define VERSION 2.0
+#define VERSION 2.1
 #endif
 
 //pid of the controlled process
@@ -96,9 +99,21 @@ int nice_lim;
 // number of CPUs we detected
 int NCPU;
 
+// quiet mode
+int quiet = FALSE;
+
+// What signal should we send to the watched process
+// when cpulimit exits?
+int send_signal = SIGCONT;
+
 //reverse byte search
 // void *memrchr(const void *s, int c, size_t n);
 
+#define MAX_SIGNAL 7
+const char *SIGNAL_NAME[MAX_SIGNAL] = { "SIGHUP", "SIGINT", "SIGQUIT", 
+                                 "SIGKILL", "SIGTERM", "SIGSTOP", "SIGCONT" };
+const int SIGNAL_VALUE[MAX_SIGNAL] = { SIGHUP, SIGINT, SIGQUIT,
+                                  SIGKILL, SIGTERM, SIGSTOP, SIGCONT };
 
 
 //return ta-tb in microseconds (no overflow checks!)
@@ -116,7 +131,7 @@ int Check_Us(pid_t target_pid)
    this_pid = getpid();
    if (this_pid == target_pid)
    {
-      printf("We cannot throttle ourselves.\n");
+      fprintf(stderr, "We cannot throttle ourselves.\n");
       exit(7);
    }
    return TRUE;
@@ -178,7 +193,7 @@ int waitforpid(int pid) {
 				exit(2);
 			}
 			else {
-				printf("Warning: no target process found. Waiting for it...\n");
+				fprintf(stderr, "Warning: no target process found. Waiting for it...\n");
 			}
 		}
 
@@ -187,7 +202,8 @@ int waitforpid(int pid) {
 	}
 
 done:
-	printf("Process %d detected\n",pid);
+    if (!quiet)
+    	printf("Process %d detected\n",pid);
 	//now set high priority, if possible
 	// if (setpriority(PRIO_PROCESS,getpid(),-20)!=0) {
         /*
@@ -282,7 +298,7 @@ int getpidof(const char *process) {
 				exit(2);
 			}
 			else {
-				printf("Warning: no target process found. Waiting for it...\n");
+				fprintf(stderr, "Warning: no target process found. Waiting for it...\n");
 			}
 		}
 
@@ -291,7 +307,8 @@ int getpidof(const char *process) {
 	}
 
 done:
-	printf("Process %d detected\n",pid);
+    if (!quiet)
+    	printf("Process %d detected\n",pid);
 	//now set high priority, if possible
 	// if (setpriority(PRIO_PROCESS,getpid(),-20)!=0) {
         /*
@@ -306,8 +323,8 @@ done:
 
 //SIGINT and SIGTERM signal handler
 void quit(int sig) {
-	//let the process continue if it's stopped
-	kill(pid,SIGCONT);
+	//let the process continue if we are stopped
+	kill(pid, send_signal);
 	printf("Exiting...\n");
 	exit(0);
 }
@@ -325,7 +342,7 @@ int getjiffies(int pid)
    my_kernel = kvm_open(0, 0, 0, O_RDONLY, "kvm_open");
    if (! my_kernel)
    {
-      printf("Error opening kernel vm. You should be running as root.\n");
+      fprintf(stderr, "Error opening kernel vm. You should be running as root.\n");
       return -1;
    }
 
@@ -499,11 +516,14 @@ void print_usage(FILE *stream,int exit_code) {
 	fprintf(stream, "      -l, --limit=N      percentage of cpu allowed from 1 up.\n");
         fprintf(stream, "                         Usually 1 - %d00, but can be higher\n", NCPU);
         fprintf(stream, "                         on multi-core CPUs (mandatory)\n");
+        fprintf(stream, "      -q, --quiet        run in quiet mode (only print errors).\n");
         fprintf(stream, "      -k, --kill         kill processes going over their limit\n");
         fprintf(stream, "                         instead of just throttling them.\n");
         fprintf(stream, "      -r, --restore      Restore processes after they have\n");
         fprintf(stream, "                         been killed. Works with the -k flag.\n");
-
+        fprintf(stream, "      -s, --signal=SIG   Send this signal to the watched process when cpulimit exits.\n");
+        fprintf(stream, "                         Signal should be specificed as a number or \n");
+        fprintf(stream, "                         SIGTERM, SIGCONT, SIGSTOP, etc. SIGCONT is the default.\n");
 	fprintf(stream, "      -v, --verbose      show control statistics\n");
 	fprintf(stream, "      -z, --lazy         exit if there is no suitable target process,\n");
         fprintf(stream, "                         or if it dies\n");
@@ -526,6 +546,33 @@ int get_ncpu()
 }
 
 
+// This function attempts to figure out what signal we should send
+// target processes based on a command line paramter. First we check
+// for text such as SIGINT, SIGCONT, SIGSTOP, etc. If no match is found
+// then we assume the value given is a number and use that.
+int Translate_Signal(char *my_signal)
+{
+    int signal_value;
+    int index = 0, found = FALSE;
+    // first check to see if we were passed a string
+    while ( (index < MAX_SIGNAL) && (! found) )
+    {
+        if (! strcmp(my_signal, SIGNAL_NAME[index]) )
+        {
+            found = TRUE;
+            signal_value = SIGNAL_VALUE[index];
+        }
+        else
+           index++;
+    }
+
+    // no value found, try a number
+    if (! found)
+       signal_value = atoi(my_signal);
+
+    return signal_value;
+}
+
 
 
 int main(int argc, char **argv) {
@@ -538,7 +585,7 @@ int main(int argc, char **argv) {
 	//parse arguments
 	int next_option;
 	/* A string listing valid short options letters. */
-	const char* short_options="p:e:P:l:c:bkrvzh";
+	const char* short_options="p:e:P:l:c:s:bqkrvzh";
 	/* An array describing valid long options. */
 	const struct option long_options[] = {
 		{ "pid", required_argument, NULL, 'p' },
@@ -546,10 +593,12 @@ int main(int argc, char **argv) {
 		{ "path", required_argument, NULL, 'P' },
 		{ "limit", required_argument, NULL, 'l' },
                 { "background", no_argument, NULL, 'b' },
+                { "quiet", no_argument, NULL, 'q' },
 		{ "verbose", no_argument, NULL, 'v' },
 		{ "lazy", no_argument, NULL, 'z' },
 		{ "help", no_argument, NULL, 'h' },
                 { "cpu", required_argument, NULL, 'c'},
+                { "signal", required_argument, NULL, 's'},
 		{ NULL, 0, NULL, 0 }
 	};
 	//argument variables
@@ -602,6 +651,14 @@ int main(int argc, char **argv) {
                                 NCPU = atoi(optarg);
                                 last_known_argument += 2;
                                 break;
+                        case 's':
+                                send_signal = Translate_Signal(optarg);
+                                if ( (send_signal < 1) || (send_signal > 35) )
+                                {
+                                    fprintf(stderr, "Specified exit signal is not recognized or not within bounds (1-35). Using SIGCONT.\n");
+                                    send_signal = SIGCONT;
+                                }
+                                last_known_argument += 2;
                         case 'k':
                                 kill_process = TRUE;
                                 last_known_argument++;
@@ -615,6 +672,10 @@ int main(int argc, char **argv) {
 				verbose = TRUE;
                                 last_known_argument++;
 				break;
+                        case 'q':
+                                quiet = TRUE;
+                                last_known_argument++;
+                                break;
 			case 'z':
 				lazy = TRUE;
                                 last_known_argument++;
@@ -841,7 +902,8 @@ wait_for_process:
 		//estimate how much the controlled process is using the cpu in its working interval
 		struct cpu_usage cu;
 		if (compute_cpu_usage(pid,workingtime,&cu)==-1) {
-			fprintf(stderr,"Process %d dead!\n",pid);
+            if (!quiet)
+    			fprintf(stderr,"Process %d dead!\n",pid);
 			if (lazy) exit(2);
 			//wait until our process appears
 			goto wait_for_process;		
@@ -883,7 +945,8 @@ wait_for_process:
                         // printf("Continue\n");
 			//resume process
 			if (kill(pid,SIGCONT)!=0) {
-				fprintf(stderr,"Process %d dead!\n",pid);
+                if (!quiet)
+    				fprintf(stderr,"Process %d dead!\n",pid);
 				if (lazy) exit(2);
 				//wait until our process appears
 				goto wait_for_process;
@@ -930,7 +993,8 @@ wait_for_process:
                      if (kill_process)
                      {
                          kill(pid, SIGKILL);
-                         fprintf(stderr, "Process %d killed.\n", pid);
+                         if (!quiet)
+                             fprintf(stderr, "Process %d killed.\n", pid);
                          if ( (lazy) && (! restore_process) ) 
                               exit(2);
                          // restart killed process
@@ -967,7 +1031,8 @@ wait_for_process:
                         // printf("Stop\n");
 			//stop process, it has worked enough
 			if (kill(pid,SIGSTOP)!=0) {
-				fprintf(stderr,"Process %d dead!\n", pid);
+                if (!quiet)
+    				fprintf(stderr,"Process %d dead!\n", pid);
 				if (lazy) exit(2);
 				//wait until our process appears
 				goto wait_for_process;
